@@ -11,13 +11,27 @@
     library.
 
 
-    :copyright: (c) 2009 by the Werkzeug Team, see AUTHORS for more details.
+    :copyright: (c) 2010 by the Werkzeug Team, see AUTHORS for more details.
     :license: BSD, see LICENSE for more details.
 """
 from types import ModuleType
 import sys
 
+# This import magic raises concerns quite often which is why the implementation
+# and motiviation is explained here in detail now.
+#
+# The majority of the functions and classes provided by Werkzeug work on the
+# HTTP and WSGI layer.  There is no useful grouping for those which is why
+# they are all importable from "werkzeug" instead of the modules where they are
+# implemented.  The downside of that is, that now everything would be loaded at
+# once, even if unused.
+#
+# The implementation of a lazy-loading module in this file replaces the
+# werkzeug package when imported from within.  Attribute access to the werkzeug
+# module will then lazily import from the modules that implement the objects.
 
+
+# import mapping to objects in other modules
 all_by_module = {
     'werkzeug.debug':       ['DebuggedApplication'],
     'werkzeug.local':       ['Local', 'LocalManager', 'LocalProxy'],
@@ -27,24 +41,24 @@ all_by_module = {
                              'run_wsgi_app'],
     'werkzeug.testapp':     ['test_app'],
     'werkzeug.exceptions':  ['abort', 'Aborter'],
-    'werkzeug.utils':       ['escape', 'url_quote',
-                             'environ_property', 'cookie_date', 'http_date',
-                             'url_encode', 'url_quote_plus', 'url_fix',
-                             'get_host', 'responder',
-                             'SharedDataMiddleware', 'ClosingIterator',
-                             'FileStorage', 'url_unquote_plus', 'url_decode',
-                             'url_unquote', 'get_current_url', 'redirect',
-                             'append_slash_redirect',
+    'werkzeug.urls':        ['url_decode', 'url_encode', 'url_quote',
+                             'url_quote_plus', 'url_unquote',
+                             'url_unquote_plus', 'url_fix', 'Href',
+                             'iri_to_uri', 'uri_to_iri'],
+    'werkzeug.formparser':  ['parse_form_data'],
+    'werkzeug.utils':       ['escape', 'environ_property', 'cookie_date',
+                             'http_date', 'append_slash_redirect', 'redirect',
                              'cached_property', 'import_string',
                              'dump_cookie', 'parse_cookie', 'unescape',
-                             'format_string', 'Href', 'DispatcherMiddleware',
-                             'find_modules', 'header_property', 'html',
-                             'xhtml', 'HTMLBuilder', 'parse_form_data',
+                             'format_string', 'find_modules', 'header_property',
+                             'html', 'xhtml', 'HTMLBuilder',
                              'validate_arguments', 'ArgumentValidationError',
-                             'bind_arguments', 'FileWrapper', 'wrap_file',
-                             'pop_path_info', 'peek_path_info',
-                             'LimitedStream', 'make_line_iter',
-                             'secure_filename'],
+                             'bind_arguments', 'secure_filename'],
+    'werkzeug.wsgi':        ['get_current_url', 'get_host', 'pop_path_info',
+                             'peek_path_info', 'SharedDataMiddleware',
+                             'DispatcherMiddleware', 'ClosingIterator',
+                             'FileWrapper', 'make_line_iter', 'LimitedStream',
+                             'responder', 'wrap_file', 'extract_path_info'],
     'werkzeug.datastructures': ['MultiDict', 'CombinedMultiDict', 'Headers',
                              'EnvironHeaders', 'ImmutableList',
                              'ImmutableDict', 'ImmutableMultiDict',
@@ -53,7 +67,8 @@ all_by_module = {
                              'LanguageAccept', 'RequestCacheControl',
                              'ResponseCacheControl', 'ETags', 'HeaderSet',
                              'WWWAuthenticate', 'Authorization',
-                             'CacheControl', 'FileMultiDict', 'CallbackDict'],
+                             'FileMultiDict', 'CallbackDict', 'FileStorage',
+                             'OrderedMultiDict', 'ImmutableOrderedMultiDict'],
     'werkzeug.useragents':  ['UserAgent'],
     'werkzeug.http':        ['parse_etags', 'parse_date', 'parse_cache_control_header',
                              'is_resource_modified', 'parse_accept_header',
@@ -78,6 +93,7 @@ all_by_module = {
     'werkzeug._internal':   ['_easteregg']
 }
 
+# modules that should be imported when accessed as attributes of werkzeug
 attribute_modules = dict.fromkeys(['exceptions', 'routing', 'script'])
 
 
@@ -87,6 +103,11 @@ for module, items in all_by_module.iteritems():
         object_origins[item] = module
 
 
+#: the cached version of the library.  We get the distribution from
+#: pkg_resources the first time this attribute is accessed.  Because
+#: this operation is quite slow it speeds up importing a lot.
+version = None
+
 class module(ModuleType):
     """Automatically import objects from the modules."""
 
@@ -100,15 +121,28 @@ class module(ModuleType):
             __import__('werkzeug.' + name)
         return ModuleType.__getattribute__(self, name)
 
+    def __dir__(self):
+        """Just show what we want to show."""
+        result = list(new_module.__all__)
+        result.extend(('__file__', '__path__', '__doc__', '__all__',
+                       '__docformat__', '__name__', '__path__',
+                       '__package__', '__version__'))
+        return result
+
+    @property
+    def __version__(self):
+        global version
+        if version is None:
+            try:
+                version = __import__('pkg_resources') \
+                          .get_distribution('Werkzeug').version
+            except:
+                version = 'unknown'
+        return version
 
 # keep a reference to this module so that it's not garbage collected
 old_module = sys.modules['werkzeug']
 
-# figure out the version
-try:
-    version = __import__('pkg_resources').get_distribution('Werkzeug').version
-except:
-    version = 'unknown'
 
 # setup the new module and patch it into the dict of loaded modules
 new_module = sys.modules['werkzeug'] = module('werkzeug')
@@ -117,6 +151,5 @@ new_module.__dict__.update({
     '__path__':         __path__,
     '__doc__':          __doc__,
     '__all__':          tuple(object_origins) + tuple(attribute_modules),
-    '__docformat__':    'restructuredtext en',
-    '__version__':      version
+    '__docformat__':    'restructuredtext en'
 })
