@@ -1,12 +1,18 @@
 # -*- coding: utf-8 -*-
-"""
+r"""
     werkzeug.contrib.sessions
     ~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    This module contains some helper classes that helps one to add session
-    support to a python WSGI application.
+    This module contains some helper classes that help one to add session
+    support to a python WSGI application.  For full client-side session
+    storage see :mod:`~werkzeug.contrib.securecookie` which implements a
+    secure, client-side session storage.
 
-    Example::
+
+    Application Integration
+    =======================
+
+    ::
 
         from werkzeug.contrib.sessions import SessionMiddleware, \
              FilesystemSessionStore
@@ -31,7 +37,7 @@
 
         def application(environ, start_response):
             request = Request(environ)
-            sid = request.cookie.get('cookie_name')
+            sid = request.cookies.get('cookie_name')
             if sid is None:
                 request.session = session_store.new()
             else:
@@ -42,24 +48,25 @@
                 response.set_cookie('cookie_name', request.session.sid)
             return response(environ, start_response)
 
-
-    :copyright: 2007 by Armin Ronacher.
+    :copyright: (c) 2010 by the Werkzeug Team, see AUTHORS for more details.
     :license: BSD, see LICENSE for more details.
 """
 import re
 import os
+import tempfile
 from os import path
 from time import time
-from random import Random, random
+from random import random
 try:
     from hashlib import sha1
 except ImportError:
     from sha import new as sha1
 from cPickle import dump, load, HIGHEST_PROTOCOL
-from werkzeug.utils import ClosingIterator, dump_cookie, parse_cookie
+
+from werkzeug import ClosingIterator, dump_cookie, parse_cookie, CallbackDict
 
 
-_sha1_re = re.compile(r'^[a-fA-F0-9]{40}$')
+_sha1_re = re.compile(r'^[a-f0-9]{40}$')
 
 
 def _urandom():
@@ -72,18 +79,15 @@ def generate_key(salt=None):
     return sha1('%s%s%s' % (salt, time(), _urandom())).hexdigest()
 
 
-class ModificationTrackingDict(dict):
+class ModificationTrackingDict(CallbackDict):
     __slots__ = ('modified',)
 
     def __init__(self, *args, **kwargs):
-        dict.__init__(self, *args, **kwargs)
+        def on_update(self):
+            self.modified = True
         self.modified = False
-
-    def __repr__(self):
-        return '<%s %s%s>' % (
-            self.__class__.__name__,
-            dict.__repr__(self)
-        )
+        CallbackDict.__init__(self, on_update=on_update)
+        dict.update(self, *args, **kwargs)
 
     def copy(self):
         """Create a flat copy of the dict."""
@@ -98,33 +102,9 @@ class ModificationTrackingDict(dict):
     def __copy__(self):
         return self.copy()
 
-    def call_with_modification(f):
-        def oncall(self, *args, **kw):
-            try:
-                return f(self, *args, **kw)
-            finally:
-                self.modified = True
-        try:
-            oncall.__name__ = f.__name__
-            oncall.__doc__ = f.__doc__
-            oncall.__module__ = f.__module__
-        except:
-            pass
-        return oncall
-
-    __setitem__ = call_with_modification(dict.__setitem__)
-    __delitem__ = call_with_modification(dict.__delitem__)
-    clear = call_with_modification(dict.clear)
-    pop = call_with_modification(dict.pop)
-    popitem = call_with_modification(dict.popitem)
-    setdefault = call_with_modification(dict.setdefault)
-    update = call_with_modification(dict.update)
-    del call_with_modification
-
 
 class Session(ModificationTrackingDict):
-    """
-    Subclass of a dict that keeps track of direct object changes.  Changes
+    """Subclass of a dict that keeps track of direct object changes.  Changes
     in mutable structures are not tracked, for those you have to set
     `modified` to `True` by hand.
     """
@@ -142,16 +122,32 @@ class Session(ModificationTrackingDict):
             self.should_save and '*' or ''
         )
 
+    @property
     def should_save(self):
-        """True if the session should be saved."""
-        return self.modified or self.new
-    should_save = property(should_save)
+        """True if the session should be saved.
+
+        .. versionchanged:: 0.6
+           By default the session is now only saved if the session is
+           modified, not if it is new like it was before.
+        """
+        return self.modified
+
+    def __repr__(self):
+        return '<%s %s %s%s>' % (
+            self.__class__.__name__,
+            self.sid,
+            dict.__repr__(self),
+            self.should_save and '*' or ''
+        )
 
 
 class SessionStore(object):
     """Baseclass for all session stores.  The Werkzeug contrib module does not
-    implement any useful stores beside the filesystem store, application
+    implement any useful stores besides the filesystem store, application
     developers are encouraged to create their own stores.
+
+    :param session_class: The session class to use.  Defaults to
+                          :class:`Session`.
     """
 
     def __init__(self, session_class=None):
@@ -190,51 +186,121 @@ class SessionStore(object):
         return self.session_class({}, sid, True)
 
 
+#: used for temporary files by the filesystem session store
+_fs_transaction_suffix = '.__wz_sess'
+
+
 class FilesystemSessionStore(SessionStore):
-    """Simple example session store that saves session on the filesystem like
+    """Simple example session store that saves sessions in the filesystem like
     PHP does.
+
+    .. versionchanged:: 0.6
+       `renew_missing` was added.  Previously this was considered `True`,
+       now the default changed to `False` and it can be explicitly
+       deactivated.
+
+    :param path: the path to the folder used for storing the sessions.
+                 If not provided the default temporary directory is used.
+    :param filename_template: a string template used to give the session
+                              a filename.  ``%s`` is replaced with the
+                              session id.
+    :param session_class: The session class to use.  Defaults to
+                          :class:`Session`.
+    :param renew_missing: set to `True` if you want the store to
+                          give the user a new sid if the session was
+                          not yet saved.
     """
 
     def __init__(self, path=None, filename_template='werkzeug_%s.sess',
-                 session_class=Session):
+                 session_class=None, renew_missing=False, mode=0644):
         SessionStore.__init__(self, session_class)
         if path is None:
-            from tempfile import gettempdir
             path = gettempdir()
         self.path = path
+        if isinstance(filename_template, unicode):
+            filename_template = filename_template.encode(
+                sys.getfilesystemencoding() or 'utf-8')
+        assert not filename_template.endswith(_fs_transaction_suffix), \
+            'filename templates may not end with %s' % _fs_transaction_suffix
         self.filename_template = filename_template
+        self.renew_missing = renew_missing
+        self.mode = mode
 
     def get_session_filename(self, sid):
+        if isinstance(sid, unicode):
+            sid = sid.encode('utf-8')
         return path.join(self.path, self.filename_template % sid)
 
     def save(self, session):
-        f = file(self.get_session_filename(session.sid), 'wb')
-        try:
-            dump(dict(session), f, HIGHEST_PROTOCOL)
-        finally:
-            f.close()
+        def _dump(filename):
+            f = file(filename, 'wb')
+            try:
+                dump(dict(session), f, HIGHEST_PROTOCOL)
+            finally:
+                f.close()
+        fn = self.get_session_filename(session.sid)
+        if os.name == 'posix':
+            td, tmp = tempfile.mkstemp(suffix=_fs_transaction_suffix,
+                                       dir=self.path)
+            _dump(tmp)
+            try:
+                os.rename(tmp, fn)
+            except (IOError, OSError):
+                pass
+            os.chmod(fn, self.mode)
+        else:
+            _dump(fn)
+            try:
+                os.chmod(fn, self.mode)
+            except OSError:
+                # maybe some platforms fail here, have not found
+                # any that do thought.
+                pass
 
     def delete(self, session):
         fn = self.get_session_filename(session.sid)
         try:
-            # Late import because Google Appengine won't allow os.unlink
-            from os import unlink
-            unlink(fn)
+            os.unlink(fn)
         except OSError:
             pass
 
     def get(self, sid):
-        fn = self.get_session_filename(sid)
-        if not self.is_valid_key(sid) or not path.exists(fn):
+        if not self.is_valid_key(sid):
             return self.new()
+        try:
+            f = open(self.get_session_filename(sid), 'rb')
+        except IOError:
+            if self.renew_missing:
+                return self.new()
+            data = {}
         else:
-            f = file(fn, 'rb')
             try:
-                data = load(f)
+                try:
+                    data = load(f)
+                except Exception:
+                    data = {}
             finally:
                 f.close()
         return self.session_class(data, sid, False)
 
+    def list(self):
+        """Lists all sessions in the store.
+
+        .. versionadded:: 0.6
+        """
+        before, after = self.filename_template.split('%s', 1)
+        filename_re = re.compile(r'%s(.{5,})%s$' % (re.escape(before),
+                                                    re.escape(after)))
+        result = []
+        for filename in os.listdir(self.path):
+            #: this is a session that is still being saved.
+            if filename.endswith(_fs_transaction_suffix):
+                continue
+            match = filename_re.match(filename)
+            if match is not None:
+                result.append(match.group(1))
+        return result
+
 
 class SessionMiddleware(object):
     """A simple middleware that puts the session object of a store provided
@@ -245,17 +311,17 @@ class SessionMiddleware(object):
     fast as sessions managed by the application itself and will put a key into
     the WSGI environment only relevant for the application which is against
     the concept of WSGI.
+
+    The cookie parameters are the same as for the :func:`~werkzeug.dump_cookie`
+    function just prefixed with ``cookie_``.  Additionally `max_age` is
+    called `cookie_age` and not `cookie_max_age` because of backwards
+    compatibility.
     """
 
     def __init__(self, app, store, cookie_name='session_id',
                  cookie_age=None, cookie_expires=None, cookie_path='/',
                  cookie_domain=None, cookie_secure=None,
                  cookie_httponly=False, environ_key='werkzeug.session'):
-        """The cookie parameters are the same as for the `dump_cookie`
-        function just prefixed with "cookie_".  Additionally "max_age" is
-        called "cookie_age" and not "cookie_max_age" because of backwards
-        compatibility.
-        """
         self.app = app
         self.store = store
         self.cookie_name = cookie_name
