@@ -3,68 +3,9 @@
     werkzeug.local
     ~~~~~~~~~~~~~~
 
-    Sooner or later you have some things you want to have in every single view
-    or helper function or whatever.  In PHP the way to go are global
-    variables.  However that is not possible in WSGI applications without a
-    major drawback:  As soon as you operate on the global namespace your
-    application is not thread safe any longer.
+    This module implements context-local objects.
 
-    The python standard library comes with a utility called "thread locals".
-    A thread local is a global object where you can put stuff on and get back
-    later in a thread safe way.  That means whenever you set or get an object
-    to / from a thread local object the thread local object checks in which
-    thread you are and delivers the correct value.
-
-    This however has a few disadvantages.  For example beside threads there
-    are other ways to handle concurrency in Python.  A very popular approach
-    are greenlets.  Also, whether every request gets its own thread is not
-    guaranteed in WSGI.  It could be that a request is reusing a thread from
-    before and data is left in the thread local object.
-
-
-    Nutshell
-    --------
-
-    Here a simple example how you can use werkzeug.local::
-
-        from werkzeug import Local, LocalManager
-
-        local = Local()
-        local_manager = LocalManager([local])
-
-        def application(environ, start_response):
-            local.request = request = Request(environ)
-            ...
-
-        application = local_manager.make_middleware(application)
-
-    Now what this code does is binding request to `local.request`.  Every
-    other piece of code executed after this assignment in the same context can
-    safely access local.request and will get the same request object.  The
-    `make_middleware` method on the local manager ensures that everything is
-    cleaned up after the request.
-
-    The same context means the same greenlet (if you're using greenlets) in
-    the same thread and same process.
-
-    If a request object is not yet set on the local object and you try to
-    access it you will get an `AttributeError`.  You can use `getattr` to avoid
-    that::
-
-        def get_request():
-            return getattr(local, 'request', None)
-
-    This will try to get the request or return `None` if the request is not
-    (yet?) available.
-
-    Note that local objects cannot manage themselves, for that you need a local
-    manager.  You can pass a local manager multiple locals or add additionals
-    later by appending them to `manager.locals` and everytime the manager
-    cleans up it will clean up all the data left in the locals for this
-    context.
-
-
-    :copyright: 2007-2008 by Armin Ronacher.
+    :copyright: (c) 2010 by the Werkzeug Team, see AUTHORS for more details.
     :license: BSD, see LICENSE for more details.
 """
 try:
@@ -72,19 +13,20 @@ try:
     get_current_greenlet = greenlet.getcurrent
     del greenlet
 except:
-    # catch all, py.* fails in so many different errors.
+    # catch all, py.* fails with so many different errors.
     get_current_greenlet = int
 try:
     from thread import get_ident as get_current_thread, allocate_lock
 except ImportError:
     from dummy_thread import get_ident as get_current_thread, allocate_lock
-from werkzeug.utils import ClosingIterator
+
+from werkzeug.wsgi import ClosingIterator
 from werkzeug._internal import _patch_wrapper
 
 
 # get the best ident function.  if greenlets are not installed we can
-# savely just use the builtin thread function and save a python methodcall
-# and the cost of caculating a hash.
+# safely just use the builtin thread function and save a python methodcall
+# and the cost of calculating a hash.
 if get_current_greenlet is int:
     get_ident = get_current_thread
 else:
@@ -254,7 +196,7 @@ class LocalProxy(object):
 
     def __unicode__(self):
         try:
-            return unicode(self.__current_oject)
+            return unicode(self.__current_object)
         except RuntimeError:
             return repr(self)
 
