@@ -30,7 +30,7 @@
     ... ], default_subdomain='www')
 
     If the application doesn't use subdomains it's perfectly fine to not set
-    the default subdomain and use the `Subdomain` rule factory.  The endpoint
+    the default subdomain and not use the `Subdomain` rule factory.  The endpoint
     in the rules can be anything, for example import paths or unique
     identifiers.  The WSGI application can use those endpoints to get the
     handler for that URL.  It doesn't have to be a string at all but it's
@@ -92,15 +92,17 @@
     method is raised.
 
 
-    :copyright: (c) 2009 by the Werkzeug Team, see AUTHORS for more details.
+    :copyright: (c) 2010 by the Werkzeug Team, see AUTHORS for more details.
                              Thomas Johansson.
     :license: BSD, see LICENSE for more details.
 """
 import re
+from pprint import pformat
 from urlparse import urljoin
 from itertools import izip
 
-from werkzeug.utils import url_encode, url_quote, redirect, format_string
+from werkzeug.urls import url_encode, url_quote
+from werkzeug.utils import redirect, format_string
 from werkzeug.exceptions import HTTPException, NotFound, MethodNotAllowed
 
 
@@ -168,8 +170,8 @@ def get_converter(map, name, args):
 
 
 class RoutingException(Exception):
-    """Special exceptions that require the application to redirect, notifies
-    him about missing urls etc.
+    """Special exceptions that require the application to redirect, notifying
+    about missing urls, etc.
 
     :internal:
     """
@@ -177,7 +179,7 @@ class RoutingException(Exception):
 
 class RequestRedirect(HTTPException, RoutingException):
     """Raise if the map requests a redirect. This is for example the case if
-    `strict_slashes` are activated and an url that requires a leading slash.
+    `strict_slashes` are activated and an url that requires a trailing slash.
 
     The attribute `new_url` contains the absolute destination url.
     """
@@ -240,7 +242,7 @@ class Subdomain(RuleFactory):
         ])
 
     All the rules except for the ``'#select_language'`` endpoint will now
-    listen on a two letter long subdomain that helds the language code
+    listen on a two letter long subdomain that holds the language code
     for the current request.
     """
 
@@ -251,6 +253,7 @@ class Subdomain(RuleFactory):
     def get_rules(self, map):
         for rulefactory in self.rules:
             for rule in rulefactory.get_rules(map):
+                rule = rule.empty()
                 rule.subdomain = self.subdomain
                 yield rule
 
@@ -276,6 +279,7 @@ class Submount(RuleFactory):
     def get_rules(self, map):
         for rulefactory in self.rules:
             for rule in rulefactory.get_rules(map):
+                rule = rule.empty()
                 rule.rule = self.path + rule.rule
                 yield rule
 
@@ -300,6 +304,7 @@ class EndpointPrefix(RuleFactory):
     def get_rules(self, map):
         for rulefactory in self.rules:
             for rule in rulefactory.get_rules(map):
+                rule = rule.empty()
                 rule.endpoint = self.prefix + rule.endpoint
                 yield rule
 
@@ -389,7 +394,7 @@ class Rule(RuleFactory):
     `endpoint`
         The endpoint for this rule. This can be anything. A reference to a
         function, a string, a number etc.  The preferred way is using a string
-        as because the endpoint is used for URL generation.
+        because the endpoint is used for URL generation.
 
     `defaults`
         An optional dict with defaults for other rules with the same endpoint.
@@ -494,7 +499,10 @@ class Rule(RuleFactory):
     def empty(self):
         """Return an unbound copy of this rule.  This can be useful if you
         want to reuse an already bound URL for another map."""
-        return Rule(self.rule, self.defaults, self.subdomain, self.methods,
+        defaults = None
+        if self.defaults is not None:
+            defaults = dict(self.defaults)
+        return Rule(self.rule, defaults, self.subdomain, self.methods,
                     self.build_only, self.endpoint, self.strict_slashes,
                     self.redirect_to)
 
@@ -581,7 +589,7 @@ class Rule(RuleFactory):
                     result.update(self.defaults)
                 return result
 
-    def build(self, values):
+    def build(self, values, append_unknown=True):
         """Assembles the relative url for that rule and the subdomain.
         If building doesn't work for some reasons `None` is returned.
 
@@ -601,13 +609,14 @@ class Rule(RuleFactory):
                 add(data)
         subdomain, url = (u''.join(tmp)).split('|', 1)
 
-        query_vars = {}
-        for key in set(values) - processed:
-            query_vars[key] = unicode(values[key])
-        if query_vars:
-            url += '?' + url_encode(query_vars, self.map.charset,
-                                    sort=self.map.sort_parameters,
-                                    key=self.map.sort_key)
+        if append_unknown:
+            query_vars = {}
+            for key in set(values) - processed:
+                query_vars[key] = unicode(values[key])
+            if query_vars:
+                url += '?' + url_encode(query_vars, self.map.charset,
+                                        sort=self.map.sort_parameters,
+                                        key=self.map.sort_key)
 
         return subdomain, url
 
@@ -753,7 +762,7 @@ class BaseConverter(object):
 
 class UnicodeConverter(BaseConverter):
     """This converter is the default converter and accepts any string but
-    only one one path segment.  Thus the string can not include a slash.
+    only one path segment.  Thus the string can not include a slash.
 
     This is the default validator.
 
@@ -809,10 +818,6 @@ class PathConverter(BaseConverter):
         Rule('/<path:wikipage>/edit')
 
     :param map: the :class:`Map`.
-    :param minlength: the minimum length of the string.  Must be greater
-                      or equal 1.
-    :param maxlength: the maximum length of the string.
-    :param length: the exact length of the string.
     """
     regex = '[^/].*?'
     is_greedy = True
@@ -889,7 +894,7 @@ class Map(object):
     parameters.  Some of the configuration values are only stored on the
     `Map` instance since those affect all rules, others are just defaults
     and can be overridden for each rule.  Note that you have to specify all
-    arguments besides the `rules` as keywords arguments!
+    arguments besides the `rules` as keyword arguments!
 
     :param rules: sequence of url rules for this map.
     :param default_subdomain: The default subdomain for rules without a
@@ -910,6 +915,10 @@ class Map(object):
         `sort_parameters` and `sort_key` was added.
     """
 
+    #: .. versionadded:: 0.6
+    #:    a dict of default converters to be used.
+    default_converters = None
+
     def __init__(self, rules=None, default_subdomain='', charset='utf-8',
                  strict_slashes=True, redirect_defaults=True,
                  converters=None, sort_parameters=False, sort_key=None):
@@ -922,7 +931,7 @@ class Map(object):
         self.strict_slashes = strict_slashes
         self.redirect_defaults = redirect_defaults
 
-        self.converters = DEFAULT_CONVERTERS.copy()
+        self.converters = self.default_converters.copy()
         if converters:
             self.converters.update(converters)
 
@@ -1002,7 +1011,7 @@ class Map(object):
 
     def bind_to_environ(self, environ, server_name=None, subdomain=None):
         """Like :meth:`bind` but you can pass it an WSGI environment and it
-        will fetch the information from that directory.  Note that because of
+        will fetch the information from that dictionary.  Note that because of
         limitations in the protocol there is no way to get the current
         subdomain and real `server_name` from the environment.  If you don't
         provide it, Werkzeug will use `SERVER_NAME` and `SERVER_PORT` (or
@@ -1017,7 +1026,7 @@ class Map(object):
 
         If the object passed as environ has an environ attribute, the value of
         this attribute is used instead.  This allows you to pass request
-        objects.  Additionally `PATH_INFO` added as a default ot the
+        objects.  Additionally `PATH_INFO` added as a default of the
         :class:`MapAdapter` so that you don't have to pass the path info to
         the match method.
 
@@ -1041,15 +1050,15 @@ class Map(object):
                    in (('https', '443'), ('http', '80')):
                     server_name += ':' + environ['SERVER_PORT']
         elif subdomain is None:
-            cur_server_name = environ.get('HTTP_HOST',
-                environ['SERVER_NAME']).split(':', 1)[0].split('.')
+            wsgi_server_name = environ.get('HTTP_HOST', environ['SERVER_NAME'])
+            cur_server_name = wsgi_server_name.split(':', 1)[0].split('.')
             real_server_name = server_name.split(':', 1)[0].split('.')
             offset = -len(real_server_name)
             if cur_server_name[offset:] != real_server_name:
                 raise ValueError('the server name provided (%r) does not '
                                  'match the server name from the WSGI '
                                  'environment (%r)' %
-                                 (environ['SERVER_NAME'], server_name))
+                                 (server_name, wsgi_server_name))
             subdomain = '.'.join(filter(None, cur_server_name[:offset]))
         return Map.bind(self, server_name, environ.get('SCRIPT_NAME'),
                         subdomain, environ['wsgi.url_scheme'],
@@ -1066,6 +1075,12 @@ class Map(object):
             self._remap = False
 
 
+    def __repr__(self):
+        rules = self.iter_rules()
+        return '%s([%s])' % (self.__class__.__name__, pformat(list(rules)))
+
+
+
 class MapAdapter(object):
     """Returned by :meth:`Map.bind` or :meth:`Map.bind_to_environ` and does
     the URL matching and building based on runtime information.
@@ -1088,7 +1103,7 @@ class MapAdapter(object):
         """Does the complete dispatching process.  `view_func` is called with
         the endpoint and a dict with the values for the view.  It should
         look up the view function, call it, and return a response object
-        or WSGI application.  http exceptions are not catched by default
+        or WSGI application.  http exceptions are not caught by default
         so that applications can display nicer error messages by just
         catching them by hand.  If you want to stick with the default
         error messages you can pass it ``catch_http_exceptions=True`` and
@@ -1137,7 +1152,7 @@ class MapAdapter(object):
                 return e
             raise
 
-    def match(self, path_info=None, method=None):
+    def match(self, path_info=None, method=None, return_rule=False):
         """The usage is simple: you just pass the match method the current
         path info as well as the method (which defaults to `GET`).  The
         following things can then happen:
@@ -1148,18 +1163,19 @@ class MapAdapter(object):
           same object as `werkzeug.exceptions.NotFound`)
 
         - you receive a `MethodNotAllowed` exception that indicates that there
-          is a match for this URL but non for the current request method.
+          is a match for this URL but not for the current request method.
           This is useful for RESTful applications.
 
         - you receive a `RequestRedirect` exception with a `new_url`
           attribute.  This exception is used to notify you about a request
-          Werkzeug requests by your WSGI application.  This is for example the
+          Werkzeug requests from your WSGI application.  This is for example the
           case if you request ``/foo`` although the correct URL is ``/foo/``
           You can use the `RequestRedirect` instance as response-like object
           similar to all other subclasses of `HTTPException`.
 
-        - you get a tuple in the form ``(endpoint, arguments)`` when there is
-          a match.
+        - you get a tuple in the form ``(endpoint, arguments)`` if there is
+          a match (unless `return_rule` is True, in which case you get a tuple
+          in the form ``(rule, arguments)``)
 
         If the path info is not passed to the match method the default path
         info of the map is used (defaults to the root URL if not defined
@@ -1173,7 +1189,7 @@ class MapAdapter(object):
 
         >>> m = Map([
         ...     Rule('/', endpoint='index'),
-        ...     Rule('/downloads/', endpoint='downloads/index'), 
+        ...     Rule('/downloads/', endpoint='downloads/index'),
         ...     Rule('/downloads/<int:id>', endpoint='downloads/show')
         ... ])
         >>> urls = m.bind("example.com", "/")
@@ -1197,6 +1213,11 @@ class MapAdapter(object):
                           path info specified on binding.
         :param method: the HTTP method used for matching.  Overrides the
                        method specified on binding.
+        :param return_rule: return the rule that matched instead of just the
+                            endpoint (defaults to `False`).
+
+        .. versionadded:: 0.6
+            `return_rule` was added.
         """
         self.map.update()
         if path_info is None:
@@ -1215,7 +1236,7 @@ class MapAdapter(object):
                     self.subdomain and self.subdomain + '.' or '',
                     self.server_name,
                     self.script_name[:-1],
-                    path_info.lstrip('/')
+                    url_quote(path_info.lstrip('/'), self.map.charset)
                 )))
             if rv is None:
                 continue
@@ -1233,7 +1254,7 @@ class MapAdapter(object):
                             subdomain and subdomain + '.' or '',
                             self.server_name,
                             self.script_name[:-1],
-                            path.lstrip('/')
+                            url_quote(path.lstrip('/'), self.map.charset)
                         )))
             if rule.redirect_to is not None:
                 if isinstance(rule.redirect_to, basestring):
@@ -1250,7 +1271,10 @@ class MapAdapter(object):
                     self.server_name,
                     self.script_name
                 ), redirect_url)))
-            return rule.endpoint, rv
+            if return_rule:
+                return rule, rv
+            else:
+                return rule.endpoint, rv
         if have_match_for:
             raise MethodNotAllowed(valid_methods=list(have_match_for))
         raise NotFound()
@@ -1272,7 +1296,8 @@ class MapAdapter(object):
             return False
         return True
 
-    def build(self, endpoint, values=None, method=None, force_external=False):
+    def build(self, endpoint, values=None, method=None, force_external=False,
+              append_unknown=True):
         """Building URLs works pretty much the other way round.  Instead of
         `match` you call `build` and pass it the endpoint and a dict of
         arguments for the placeholders.
@@ -1284,7 +1309,7 @@ class MapAdapter(object):
 
         >>> m = Map([
         ...     Rule('/', endpoint='index'),
-        ...     Rule('/downloads/', endpoint='downloads/index'), 
+        ...     Rule('/downloads/', endpoint='downloads/index'),
         ...     Rule('/downloads/<int:id>', endpoint='downloads/show')
         ... ])
         >>> urls = m.bind("example.com", "/")
@@ -1312,12 +1337,18 @@ class MapAdapter(object):
         to specify the method you want to have an URL built for if you have
         different methods for the same endpoint specified.
 
+        .. versionadded:: 0.6
+           the `append_unknown` parameter was added.
+
         :param endpoint: the endpoint of the URL to build.
         :param values: the values for the URL to build.  Unhandled values are
                        appended to the URL as query parameters.
         :param method: the HTTP method for the rule if there are different
                        URLs for different methods on the same endpoint.
         :param force_external: enforce full canonical external URLs.
+        :param append_unknown: unknown parameters are appended to the generated
+                               URL as query string argument.  Disable this
+                               if you want the builder to ignore those.
         """
         self.map.update()
         method = method or self.default_method
@@ -1328,7 +1359,7 @@ class MapAdapter(object):
 
         for rule in self.map._rules_by_endpoint.get(endpoint, ()):
             if rule.suitable_for(values, method):
-                rv = rule.build(values)
+                rv = rule.build(values, append_unknown)
                 if rv is not None:
                     break
         else:
@@ -1354,3 +1385,6 @@ DEFAULT_CONVERTERS = {
     'int':              IntegerConverter,
     'float':            FloatConverter
 }
+
+from werkzeug.datastructures import ImmutableDict
+Map.default_converters = ImmutableDict(DEFAULT_CONVERTERS)
