@@ -61,7 +61,7 @@ r'''
     or as named parameters, pretty much like Python function calls.
 
 
-    :copyright: (c) 2009 by the Werkzeug Team, see AUTHORS for more details.
+    :copyright: (c) 2010 by the Werkzeug Team, see AUTHORS for more details.
     :license: BSD, see LICENSE for more details.
 '''
 import sys
@@ -87,7 +87,7 @@ converters = {
 
 
 def run(namespace=None, action_prefix='action_', args=None):
-    """Run the script.  Participating actions are looked up in the callers
+    """Run the script.  Participating actions are looked up in the caller's
     namespace if no namespace is given, otherwise in the dict provided.
     Only items that start with action_prefix are processed as actions.  If
     you want to use all items in the namespace provided as actions set
@@ -294,5 +294,6 @@ def make_runserver(app_factory, hostname='localhost', port=5000,
         from werkzeug.serving import run_simple
         app = app_factory()
         run_simple(hostname, port, app, reloader, debugger, evalex,
-                   extra_files, 1, threaded, processes, static_files)
+                   extra_files, 1, threaded, processes,
+                   static_files=static_files)
     return action
